@@ -15,6 +15,7 @@ import logging
 import os
 import os.path
 from SocketServer import ThreadingMixIn
+import signal
 import sys
 import urlparse
 
@@ -24,6 +25,7 @@ from thrift.transport import TSocket
 from thrift.transport import TTransport
 
 import flawless.lib.config
+from flawless.lib.storage import DiskStorage
 from flawless.server.api import Flawless
 from flawless.server.service import FlawlessThriftServiceHandler
 from flawless.server.service import FlawlessWebServiceHandler
@@ -86,16 +88,27 @@ class SimpleRequestHTTPHandler(BaseHTTPRequestHandler):
             self.wfile.write(ret)
 
 
-def serve(conf_path):
+def serve(conf_path, storage_cls=DiskStorage):
+    """This method starts the server. There are two processes, one is an HTTP server that shows
+    and admin interface and the second is a Thrift server that the client code calls.
+
+    Arguments:
+        `conf_path` - The path to your flawless.cfg file
+        `storage_cls` - You can pass in your own storage class that implements StorageInterface. You must implement
+                        storage_cls if you want Flawless to be horizontally scalable, since by default it will just
+                        store everything on the local disk.
+    """
+
     flawless.lib.config.init_config(conf_path)
     # Try and create datadir if it doesn't exist. For instance it might be in /tmp
     if not os.path.exists(config.data_dir_path):
         os.makedirs(config.data_dir_path)
 
-    if os.fork() == 0:
+    child_pid = os.fork()
+    if child_pid == 0:
         # Setup HTTP server
         logging.basicConfig(level=getattr(logging, config.log_level), filename=config.log_file, stream=sys.stderr)
-        handler = FlawlessWebServiceHandler()
+        handler = FlawlessWebServiceHandler(storage_cls=storage_cls)
         server = SimpleThreadedHTTPServer(('', config.http_port), SimpleRequestHTTPHandler)
         server.attach_service(handler)
         server.request_queue_size = 50
@@ -106,7 +119,7 @@ def serve(conf_path):
             server.server_close()
     else:
         # Setup Thrift server
-        handler = FlawlessThriftServiceHandler()
+        handler = FlawlessThriftServiceHandler(storage_cls=storage_cls)
         processor = Flawless.Processor(handler)
         transport = TSocket.TServerSocket(port=config.port)
         tfactory = TTransport.TFramedTransportFactory()
@@ -117,6 +130,7 @@ def serve(conf_path):
         except (KeyboardInterrupt, SystemExit):
             handler.errors_seen.sync()
             transport.close()
+            os.kill(child_pid, signal.SIGINT)
 
 
 def main():
